# -*- coding: utf-8 -*-
"""
@author: Svenja C. Hövelmann, Kiel University and Deutsches Elektronen-Synchrotron DESY

last edit: 28 February 2024

SAXS data analysis: Modelfunctions for space groups for liquid crystals.
"""
import numpy


class create_model(object):
    def __init__(self, q, spacing, spacing2 = 0, peakform="gauss"):
        self.q          = q
        self.spacing    = spacing
        self.spacing2   = spacing2
        self.peakform   = peakform

    def __call__(self):
        print("Do you need help? Use .help() instead.")
    
    def available_phases():
        print("lamellar")
        print("p6m")
        print("Primitive Im3m")
        print("Diamond Pn3m")
        print("Gyroid Ia3d")
        print("bcc Im3m")
        print("fcc Fm3m")
        print("Pm3n")
        print("Fd3m")
        
        
    def gaussian(q, amp, pos, sig, bg = 0):
        return (bg  + amp * numpy.exp(-(q-pos)**2/(2*sig**2)) )

    def lorentian(x, a1, p1, w1, bg = 0):
        return (bg + a1 /(1+((x-p1)/w1)**2) )
                
    def gaussian10(q, amp, pos, sig, bg = 0):
        return (bg  + amp[0] * numpy.exp(-(q-pos[0])**2/(2*sig[0]**2)) + bg + amp[1] * numpy.exp(-(q-pos[1])**2/(2*sig[1]**2)) 
                    + amp[2] * numpy.exp(-(q-pos[2])**2/(2*sig[2]**2)) + bg + amp[3] * numpy.exp(-(q-pos[3])**2/(2*sig[3]**2))
                    + amp[4] * numpy.exp(-(q-pos[4])**2/(2*sig[4]**2)) + bg + amp[5] * numpy.exp(-(q-pos[5])**2/(2*sig[5]**2)) 
                    + amp[6] * numpy.exp(-(q-pos[6])**2/(2*sig[6]**2)) + bg + amp[7] * numpy.exp(-(q-pos[7])**2/(2*sig[7]**2)) 
                    + amp[8] * numpy.exp(-(q-pos[8])**2/(2*sig[8]**2)) + bg + amp[9] * numpy.exp(-(q-pos[9])**2/(2*sig[9]**2))
                   )

    def lorentian10(x, a1, p1, w1, a2, p2, w2, a3, p3, w3, a4, p4, w4, a5, p5, w5, a6, p6, w6, a7, p7, w7, a8, p8, w8, a9, p9, w9, a10, p10, w10, bg = 0):
        return (bg + a1 /(1+((x-p1)/w1)**2) + a2 /(1+((x-p2)/w2)**2)
                   + a3 /(1+((x-p3)/w3)**2) + a4 /(1+((x-p4)/w4)**2)
                   + a5 /(1+((x-p5)/w5)**2) + a6 /(1+((x-p4)/w6)**2)
                   + a7 /(1+((x-p7)/w7)**2) + a8 /(1+((x-p8)/w8)**2)
                   + a9 /(1+((x-p9)/w9)**2) + a10 /(1+((x-p10)/w10)**2) )

    def set_phase(self, phase):
        self.phase = phase
    
    def set_start_parameters(self, amp = 1, sig = 0.05, bg = 0, delta = 0):       
        try:
            self.amp = numpy.zeros(10)+1*(numpy.pad(amp, (0,10-len(amp)), "constant", constant_values= 0))
        except:
            self.amp = numpy.zeros(10)+amp
        try:
            self.sig = numpy.zeros(10)+1*(numpy.pad(sig, (0,10-len(sig)), "constant", constant_values= 0))
        except:
            self.sig = numpy.zeros(10)+sig
        try:
            self.delta = numpy.zeros(10)+1*(numpy.pad(delta, (0,10-len(delta)), "constant", constant_values= 0))
        except:
            self.delta = numpy.zeros(10)+delta
        self.bg = bg
#        self.pos = self.calc_pos(phase)
        
    def calc_pos(self, phase, peaknumber = "all"):
        if phase == "lamellar":
            self.pos = (2*numpy.pi/self.spacing)*numpy.array([1,2,3,4,5,6,7,8,9,10])
        elif phase == "p6m":
            self.pos = (2*numpy.pi/self.spacing)*numpy.array([1,numpy.sqrt(3),numpy.sqrt(4),numpy.sqrt(7),numpy.sqrt(9),numpy.sqrt(12),numpy.sqrt(13),numpy.sqrt(100),numpy.sqrt(100),numpy.sqrt(100)])
        elif phase == "Pn3m":
            self.pos = (2*numpy.pi/self.spacing)*numpy.array([numpy.sqrt(2),numpy.sqrt(3),numpy.sqrt(4),numpy.sqrt(6),numpy.sqrt(8),numpy.sqrt(9),numpy.sqrt(10),numpy.sqrt(12),numpy.sqrt(14),numpy.sqrt(15)])
        elif phase == "Im3m":
            self.pos = (2*numpy.pi/self.spacing)*numpy.array([numpy.sqrt(2),numpy.sqrt(4),numpy.sqrt(6),numpy.sqrt(8),numpy.sqrt(10),numpy.sqrt(12),numpy.sqrt(14),numpy.sqrt(16),numpy.sqrt(18),numpy.sqrt(20)])
        elif phase == "Ia3d":
            self.pos = (2*numpy.pi/self.spacing)*numpy.array([numpy.sqrt(6),numpy.sqrt(8),numpy.sqrt(14),numpy.sqrt(16),numpy.sqrt(20),numpy.sqrt(22),numpy.sqrt(100),numpy.sqrt(100),numpy.sqrt(100),numpy.sqrt(100)])
        elif phase == "Fm3m":
            self.pos = (2*numpy.pi/self.spacing)*numpy.array([numpy.sqrt(3),numpy.sqrt(4),numpy.sqrt(8),numpy.sqrt(11),numpy.sqrt(12),numpy.sqrt(100),numpy.sqrt(100),numpy.sqrt(100),numpy.sqrt(100),numpy.sqrt(100)])
        elif phase == "Fd3m":
            self.pos = (2*numpy.pi/self.spacing)*numpy.array([numpy.sqrt(3),numpy.sqrt(8),numpy.sqrt(11),numpy.sqrt(16),numpy.sqrt(19),numpy.sqrt(100),numpy.sqrt(100),numpy.sqrt(100),numpy.sqrt(100),numpy.sqrt(100)])
        elif phase == "Pm3n":
            self.pos = (2*numpy.pi/self.spacing)*numpy.array([numpy.sqrt(2),numpy.sqrt(4),numpy.sqrt(5),numpy.sqrt(6),numpy.sqrt(8),numpy.sqrt(9),numpy.sqrt(10),numpy.sqrt(100),numpy.sqrt(100),numpy.sqrt(100)])
        else:
            print("Given phase is not defined. PLease chose one of the avalaible phases. Check available phases by calling the available_phases function.")
        if peaknumber == "all":     
            return self.pos
        else:
            try:
                return self.pos[peaknumber]
            except: 
                print("Please enter a number between 0 and 9 or choose \"all\".")
                
    def calc_spacing(self, phase, posarray):
        if phase == "lamellar":
            spacingarray = numpy.array([1,2,3,4,5,6,7,8,9,10])
        elif phase == "Pn3m":
            spacingarray = numpy.array([numpy.sqrt(2),numpy.sqrt(3),numpy.sqrt(4),numpy.sqrt(6),numpy.sqrt(8),numpy.sqrt(9),numpy.sqrt(10),numpy.sqrt(12),numpy.sqrt(14),numpy.sqrt(15)])
        elif phase == "Im3m":
            spacingarray = numpy.array([numpy.sqrt(2),numpy.sqrt(4),numpy.sqrt(6),numpy.sqrt(8),numpy.sqrt(10),numpy.sqrt(12),numpy.sqrt(14),numpy.sqrt(16),numpy.sqrt(18),numpy.sqrt(20)])
        elif phase == "p6m":
            spacingarray = numpy.array([1,numpy.sqrt(3),numpy.sqrt(4),numpy.sqrt(7),numpy.sqrt(9),numpy.sqrt(12),numpy.sqrt(13),numpy.sqrt(100),numpy.sqrt(100),numpy.sqrt(100)])
        else:
            spacingarray = numpy.zeros(10)
            print("Given phase is not defined. PLease chose one of the avalaible phases. Check available phases by calling the available_phases function.")
        spacings = []
        i = 0
        for pos in posarray:
            spacings = numpy.append(spacings,(2*numpy.pi/pos)*spacingarray[i])
            i += 1
        return spacings
    
#Lamellar    
    def gauss_lamellar(q, amp0, amp1, amp2, amp3, amp4, amp5, amp6, amp7, amp8, amp9, sig0, sig1, sig2, sig3, sig4, sig5, sig6, sig7, sig8, sig9, spacing, delta0, delta1, delta2, delta3, delta4, delta5, delta6, delta7, delta8, delta9, bg = 0):
        pos0, pos1, pos2, pos3, pos4, pos5, pos6, pos7, pos8, pos9 = (2*numpy.pi/spacing)*numpy.array([1,2,3,4,5,6,7,8,9,10]) + numpy.array([delta0,delta1,delta2,delta3,delta4,delta5,delta6,delta7,delta8,delta9])
        return (bg  + amp0 * numpy.exp(-(q-pos0)**2/(2*sig0**2)) + amp1 * numpy.exp(-(q-pos1)**2/(2*sig1**2)) 
                    + amp2 * numpy.exp(-(q-pos2)**2/(2*sig2**2)) + amp3 * numpy.exp(-(q-pos3)**2/(2*sig3**2))
                    + amp4 * numpy.exp(-(q-pos4)**2/(2*sig4**2)) + amp5 * numpy.exp(-(q-pos5)**2/(2*sig5**2)) 
                    + amp6 * numpy.exp(-(q-pos6)**2/(2*sig6**2)) + amp7 * numpy.exp(-(q-pos7)**2/(2*sig7**2)) 
                    + amp8 * numpy.exp(-(q-pos8)**2/(2*sig8**2)) + amp9 * numpy.exp(-(q-pos9)**2/(2*sig9**2))
                    )

#hexagonal
    def gauss_p6m(q, amp0, amp1, amp2, amp3, amp4, amp5, amp6, amp7, amp8, amp9, sig0, sig1, sig2, sig3, sig4, sig5, sig6, sig7, sig8, sig9, spacing, delta0, delta1, delta2, delta3, delta4, delta5, delta6, delta7, delta8, delta9, bg = 0):
        pos0, pos1, pos2, pos3, pos4, pos5, pos6, pos7, pos8, pos9 = (2*numpy.pi/spacing)*numpy.array([1,numpy.sqrt(3),numpy.sqrt(4),numpy.sqrt(7),numpy.sqrt(9),numpy.sqrt(12),numpy.sqrt(13),numpy.sqrt(0),numpy.sqrt(0),numpy.sqrt(0)]) + numpy.array([delta0,delta1,delta2,delta3,delta4,delta5,delta6,delta7,delta8,delta9])
        return (bg  + amp0 * numpy.exp(-(q-pos0)**2/(2*sig0**2)) + amp1 * numpy.exp(-(q-pos1)**2/(2*sig1**2)) 
                    + amp2 * numpy.exp(-(q-pos2)**2/(2*sig2**2)) + amp3 * numpy.exp(-(q-pos3)**2/(2*sig3**2))
                    + amp4 * numpy.exp(-(q-pos4)**2/(2*sig4**2)) + amp5 * numpy.exp(-(q-pos5)**2/(2*sig5**2)) 
                    + amp6 * numpy.exp(-(q-pos6)**2/(2*sig6**2)) + amp7 * numpy.exp(-(q-pos7)**2/(2*sig7**2)) 
                    + amp8 * numpy.exp(-(q-pos8)**2/(2*sig8**2)) + amp9 * numpy.exp(-(q-pos9)**2/(2*sig9**2))
                    )

#Bicontinous cubics 
    def gauss_Pn3m(q, amp0, amp1, amp2, amp3, amp4, amp5, amp6, amp7, amp8, amp9, sig0, sig1, sig2, sig3, sig4, sig5, sig6, sig7, sig8, sig9, spacing, delta0, delta1, delta2, delta3, delta4, delta5, delta6, delta7, delta8, delta9, bg=0):
        pos0, pos1, pos2, pos3, pos4, pos5, pos6, pos7, pos8, pos9 = (2*numpy.pi/spacing)*numpy.array([numpy.sqrt(2),numpy.sqrt(3),numpy.sqrt(4),numpy.sqrt(6),numpy.sqrt(8),numpy.sqrt(9),numpy.sqrt(10),numpy.sqrt(12),numpy.sqrt(14),numpy.sqrt(15)]) + numpy.array([delta0,delta1,delta2,delta3,delta4,delta5,delta6,delta7,delta8,delta9])
        return (bg  + amp0 * numpy.exp(-(q-pos0)**2/(2*sig0**2)) + amp1 * numpy.exp(-(q-pos1)**2/(2*sig1**2)) 
                    + amp2 * numpy.exp(-(q-pos2)**2/(2*sig2**2)) + amp3 * numpy.exp(-(q-pos3)**2/(2*sig3**2))
                    + amp4 * numpy.exp(-(q-pos4)**2/(2*sig4**2)) + amp5 * numpy.exp(-(q-pos5)**2/(2*sig5**2)) 
                    + amp6 * numpy.exp(-(q-pos6)**2/(2*sig6**2)) + amp7 * numpy.exp(-(q-pos7)**2/(2*sig7**2)) 
                    + amp8 * numpy.exp(-(q-pos8)**2/(2*sig8**2)) + amp9 * numpy.exp(-(q-pos9)**2/(2*sig9**2))
                    )

    def gauss_Im3m(q, amp0, amp1, amp2, amp3, amp4, amp5, amp6, amp7, amp8, amp9, sig0, sig1, sig2, sig3, sig4, sig5, sig6, sig7, sig8, sig9, spacing, delta0, delta1, delta2, delta3, delta4, delta5, delta6, delta7, delta8, delta9, bg = 0):
        pos0, pos1, pos2, pos3, pos4, pos5, pos6, pos7, pos8, pos9 = (2*numpy.pi/spacing)*numpy.array([numpy.sqrt(2),numpy.sqrt(4),numpy.sqrt(6),numpy.sqrt(8),numpy.sqrt(10),numpy.sqrt(12),numpy.sqrt(14),numpy.sqrt(16),numpy.sqrt(18),numpy.sqrt(20)]) + numpy.array([delta0,delta1,delta2,delta3,delta4,delta5,delta6,delta7,delta8,delta9])
        return (bg  + amp0 * numpy.exp(-(q-pos0)**2/(2*sig0**2)) + amp1 * numpy.exp(-(q-pos1)**2/(2*sig1**2)) 
                    + amp2 * numpy.exp(-(q-pos2)**2/(2*sig2**2)) + amp3 * numpy.exp(-(q-pos3)**2/(2*sig3**2))
                    + amp4 * numpy.exp(-(q-pos4)**2/(2*sig4**2)) + amp5 * numpy.exp(-(q-pos5)**2/(2*sig5**2)) 
                    + amp6 * numpy.exp(-(q-pos6)**2/(2*sig6**2)) + amp7 * numpy.exp(-(q-pos7)**2/(2*sig7**2)) 
                    + amp8 * numpy.exp(-(q-pos8)**2/(2*sig8**2)) + amp9 * numpy.exp(-(q-pos9)**2/(2*sig9**2))
                    )

    def gauss_Ia3d(q, amp0, amp1, amp2, amp3, amp4, amp5, amp6, amp7, amp8, amp9, sig0, sig1, sig2, sig3, sig4, sig5, sig6, sig7, sig8, sig9, spacing, delta0, delta1, delta2, delta3, delta4, delta5, delta6, delta7, delta8, delta9, bg = 0):
        pos0, pos1, pos2, pos3, pos4, pos5, pos6, pos7, pos8, pos9 = (2*numpy.pi/spacing)*numpy.array([numpy.sqrt(6),numpy.sqrt(8),numpy.sqrt(14),numpy.sqrt(16),numpy.sqrt(20),numpy.sqrt(22),numpy.sqrt(0),numpy.sqrt(0),numpy.sqrt(0),numpy.sqrt(0)]) + numpy.array([delta0,delta1,delta2,delta3,delta4,delta5,delta6,delta7,delta8,delta9])
        return (bg  + amp0 * numpy.exp(-(q-pos0)**2/(2*sig0**2)) + amp1 * numpy.exp(-(q-pos1)**2/(2*sig1**2)) 
                    + amp2 * numpy.exp(-(q-pos2)**2/(2*sig2**2)) + amp3 * numpy.exp(-(q-pos3)**2/(2*sig3**2))
                    + amp4 * numpy.exp(-(q-pos4)**2/(2*sig4**2)) + amp5 * numpy.exp(-(q-pos5)**2/(2*sig5**2)) 
                    + amp6 * numpy.exp(-(q-pos6)**2/(2*sig6**2)) + amp7 * numpy.exp(-(q-pos7)**2/(2*sig7**2)) 
                    + amp8 * numpy.exp(-(q-pos8)**2/(2*sig8**2)) + amp9 * numpy.exp(-(q-pos9)**2/(2*sig9**2))
                    )

    def gauss_Fm3m(q, amp0, amp1, amp2, amp3, amp4, amp5, amp6, amp7, amp8, amp9, sig0, sig1, sig2, sig3, sig4, sig5, sig6, sig7, sig8, sig9, spacing, delta0, delta1, delta2, delta3, delta4, delta5, delta6, delta7, delta8, delta9, bg = 0):
        pos0, pos1, pos2, pos3, pos4, pos5, pos6, pos7, pos8, pos9 = (2*numpy.pi/spacing)*numpy.array([numpy.sqrt(3),numpy.sqrt(4),numpy.sqrt(8),numpy.sqrt(11),numpy.sqrt(12),numpy.sqrt(0),numpy.sqrt(0),numpy.sqrt(0),numpy.sqrt(0),numpy.sqrt(0)]) + numpy.array([delta0,delta1,delta2,delta3,delta4,delta5,delta6,delta7,delta8,delta9])
        return (bg  + amp0 * numpy.exp(-(q-pos0)**2/(2*sig0**2)) + amp1 * numpy.exp(-(q-pos1)**2/(2*sig1**2)) 
                    + amp2 * numpy.exp(-(q-pos2)**2/(2*sig2**2)) + amp3 * numpy.exp(-(q-pos3)**2/(2*sig3**2))
                    + amp4 * numpy.exp(-(q-pos4)**2/(2*sig4**2)) + amp5 * numpy.exp(-(q-pos5)**2/(2*sig5**2)) 
                    + amp6 * numpy.exp(-(q-pos6)**2/(2*sig6**2)) + amp7 * numpy.exp(-(q-pos7)**2/(2*sig7**2)) 
                    + amp8 * numpy.exp(-(q-pos8)**2/(2*sig8**2)) + amp9 * numpy.exp(-(q-pos9)**2/(2*sig9**2))
                    )

    def gauss_Pm3n(q, amp0, amp1, amp2, amp3, amp4, amp5, amp6, amp7, amp8, amp9, sig0, sig1, sig2, sig3, sig4, sig5, sig6, sig7, sig8, sig9, spacing, delta0, delta1, delta2, delta3, delta4, delta5, delta6, delta7, delta8, delta9, bg = 0):
        pos0, pos1, pos2, pos3, pos4, pos5, pos6, pos7, pos8, pos9 = (2*numpy.pi/spacing)*numpy.array([numpy.sqrt(2),numpy.sqrt(4),numpy.sqrt(5),numpy.sqrt(6),numpy.sqrt(8),numpy.sqrt(9),numpy.sqrt(10),numpy.sqrt(0),numpy.sqrt(0),numpy.sqrt(0)]) + numpy.array([delta0,delta1,delta2,delta3,delta4,delta5,delta6,delta7,delta8,delta9])
        return (bg  + amp0 * numpy.exp(-(q-pos0)**2/(2*sig0**2)) + amp1 * numpy.exp(-(q-pos1)**2/(2*sig1**2)) 
                    + amp2 * numpy.exp(-(q-pos2)**2/(2*sig2**2)) + amp3 * numpy.exp(-(q-pos3)**2/(2*sig3**2))
                    + amp4 * numpy.exp(-(q-pos4)**2/(2*sig4**2)) + amp5 * numpy.exp(-(q-pos5)**2/(2*sig5**2)) 
                    + amp6 * numpy.exp(-(q-pos6)**2/(2*sig6**2)) + amp7 * numpy.exp(-(q-pos7)**2/(2*sig7**2)) 
                    + amp8 * numpy.exp(-(q-pos8)**2/(2*sig8**2)) + amp9 * numpy.exp(-(q-pos9)**2/(2*sig9**2))
                    )

    def gauss_Fd3m(q, amp0, amp1, amp2, amp3, amp4, amp5, amp6, amp7, amp8, amp9, sig0, sig1, sig2, sig3, sig4, sig5, sig6, sig7, sig8, sig9, spacing, delta0, delta1, delta2, delta3, delta4, delta5, delta6, delta7, delta8, delta9, bg = 0):
        pos0, pos1, pos2, pos3, pos4, pos5, pos6, pos7, pos8, pos9 = (2*numpy.pi/spacing)*numpy.array([numpy.sqrt(3),numpy.sqrt(8),numpy.sqrt(11),numpy.sqrt(16),numpy.sqrt(19),numpy.sqrt(0),numpy.sqrt(0),numpy.sqrt(0),numpy.sqrt(0),numpy.sqrt(0)]) + numpy.array([delta0,delta1,delta2,delta3,delta4,delta5,delta6,delta7,delta8,delta9])
        return (bg  + amp0 * numpy.exp(-(q-pos0)**2/(2*sig0**2)) + amp1 * numpy.exp(-(q-pos1)**2/(2*sig1**2)) 
                    + amp2 * numpy.exp(-(q-pos2)**2/(2*sig2**2)) + amp3 * numpy.exp(-(q-pos3)**2/(2*sig3**2))
                    + amp4 * numpy.exp(-(q-pos4)**2/(2*sig4**2)) + amp5 * numpy.exp(-(q-pos5)**2/(2*sig5**2)) 
                    + amp6 * numpy.exp(-(q-pos6)**2/(2*sig6**2)) + amp7 * numpy.exp(-(q-pos7)**2/(2*sig7**2)) 
                    + amp8 * numpy.exp(-(q-pos8)**2/(2*sig8**2)) + amp9 * numpy.exp(-(q-pos9)**2/(2*sig9**2))
                    )

#Bicontinous cubics Pn3m + lamellar
    def gauss_mixed_lamellar_Pn3m(q, amp0, amp1, amp2, amp3, amp4, amp5, amp6, amp7, amp8, amp9, sig0, sig1, sig2, sig3, sig4, sig5, sig6, sig7, sig8, sig9, spacing, spacing2, delta0, delta1, delta2, delta3, delta4, delta5, delta6, delta7, delta8, delta9, bg=0):
        pos0, pos1, pos2 = (2*numpy.pi/spacing)*numpy.array([1,2,3]) + numpy.array([delta0,delta1,delta2])
        pos3, pos4, pos5, pos6, pos7, pos8, pos9 = (2*numpy.pi/spacing2)*numpy.array([numpy.sqrt(2),numpy.sqrt(3),numpy.sqrt(4),numpy.sqrt(6),numpy.sqrt(8),numpy.sqrt(9),numpy.sqrt(10)]) + numpy.array([delta3,delta4,delta5,delta6,delta7,delta8,delta9])
        return (bg  + amp0 * numpy.exp(-(q-pos0)**2/(2*sig0**2)) + amp1 * numpy.exp(-(q-pos1)**2/(2*sig1**2)) 
                    + amp2 * numpy.exp(-(q-pos2)**2/(2*sig2**2)) + amp3 * numpy.exp(-(q-pos3)**2/(2*sig3**2))
                    + amp4 * numpy.exp(-(q-pos4)**2/(2*sig4**2)) + amp5 * numpy.exp(-(q-pos5)**2/(2*sig5**2)) 
                    + amp6 * numpy.exp(-(q-pos6)**2/(2*sig6**2)) + amp7 * numpy.exp(-(q-pos7)**2/(2*sig7**2)) 
                    + amp8 * numpy.exp(-(q-pos8)**2/(2*sig8**2)) + amp9 * numpy.exp(-(q-pos9)**2/(2*sig9**2))
                    )

#Hexagonal p6m + lamellar
    def gauss_mixed_p6m_Pn3m(q, amp0, amp1, amp2, amp3, amp4, amp5, amp6, amp7, amp8, amp9, sig0, sig1, sig2, sig3, sig4, sig5, sig6, sig7, sig8, sig9, spacing, spacing2, delta0, delta1, delta2, delta3, delta4, delta5, delta6, delta7, delta8, delta9, bg=0):
        pos0, pos1, pos2 = (2*numpy.pi/spacing)*numpy.array([1,numpy.sqrt(3),numpy.sqrt(4)]) + numpy.array([delta0,delta1,delta2])
        pos3, pos4, pos5, pos6, pos7, pos8, pos9 = (2*numpy.pi/spacing2)*numpy.array([numpy.sqrt(2),numpy.sqrt(3),numpy.sqrt(4),numpy.sqrt(6),numpy.sqrt(8),numpy.sqrt(9),numpy.sqrt(10)]) + numpy.array([delta3,delta4,delta5,delta6,delta7,delta8,delta9])
        return (bg  + amp0 * numpy.exp(-(q-pos0)**2/(2*sig0**2)) + amp1 * numpy.exp(-(q-pos1)**2/(2*sig1**2)) 
                    + amp2 * numpy.exp(-(q-pos2)**2/(2*sig2**2)) + amp3 * numpy.exp(-(q-pos3)**2/(2*sig3**2))
                    + amp4 * numpy.exp(-(q-pos4)**2/(2*sig4**2)) + amp5 * numpy.exp(-(q-pos5)**2/(2*sig5**2)) 
                    + amp6 * numpy.exp(-(q-pos6)**2/(2*sig6**2)) + amp7 * numpy.exp(-(q-pos7)**2/(2*sig7**2)) 
                    + amp8 * numpy.exp(-(q-pos8)**2/(2*sig8**2)) + amp9 * numpy.exp(-(q-pos9)**2/(2*sig9**2))
                    )
